@@ -26,11 +26,13 @@ import org.mule.api.registry.MuleRegistry;
 import org.mule.api.registry.RegistrationException;
 import org.mule.api.registry.Registry;
 import org.mule.api.security.SecurityManager;
+import org.mule.api.serialization.ObjectSerializer;
 import org.mule.api.store.ListableObjectStore;
 import org.mule.api.store.ObjectStoreManager;
 import org.mule.api.util.StreamCloserService;
 import org.mule.context.notification.NotificationException;
 import org.mule.context.notification.ServerNotificationManager;
+import org.mule.extension.ExtensionManager;
 import org.mule.management.stats.AllStatistics;
 import org.mule.management.stats.ProcessingTimeWatcher;
 import org.mule.util.lock.LockFactory;
@@ -188,12 +190,32 @@ public interface MuleContext extends Lifecycle
     
     ObjectStoreManager getObjectStoreManager();
 
+    ExtensionManager getExtensionManager();
+
+    /**
+     * The instance of {@link org.mule.api.serialization.ObjectSerializer}
+     * to be used to serialize/deserealize objects
+     *
+     * @return a {@link org.mule.api.serialization.ObjectSerializer}
+     * @since 3.7.0
+     */
+    ObjectSerializer getObjectSerializer();
+
     AllStatistics getStatistics();
 
     LifecycleManager getLifecycleManager();
 
     MuleRegistry getRegistry();
 
+    /**
+     * Returns a {@link Injector} capable of
+     * injecting dependencies into objects
+     *
+     * @return a {@link Injector}
+     * @since 3.7.0
+     */
+    Injector getInjector();
+
     MuleConfiguration getConfiguration();
 
     ThreadingProfile getDefaultMessageDispatcherThreadingProfile();
@@ -214,10 +236,16 @@ public interface MuleContext extends Lifecycle
      */
     public StreamCloserService getStreamCloserService();
 
-    // TODO This should ideally only be available via an Admin interface
+    /**
+     * @deprecated as of 3.7.0. This will be removed in Mule 4.0
+     */
+    @Deprecated
     void addRegistry(Registry registry);
 
-    // TODO This should ideally only be available via an Admin interface
+    /**
+     * @deprecated as of 3.7.0. This will be removed in Mule 4.0
+     */
+    @Deprecated
     void removeRegistry(Registry registry);
 
     /**
@@ -334,5 +362,6 @@ public interface MuleContext extends Lifecycle
      * @throws InterruptedException if the current thread is interrupted while waiting
      */
     boolean waitUntilStarted(int timeout) throws InterruptedException;
+
 }
 
