@@ -11,11 +11,14 @@ import org.mule.api.MuleEvent;
 import org.mule.api.MuleException;
 import org.mule.api.MuleRuntimeException;
 import org.mule.api.construct.FlowConstruct;
+import org.mule.api.construct.FlowConstructAware;
 import org.mule.api.context.MuleContextAware;
 import org.mule.api.lifecycle.Disposable;
 import org.mule.api.lifecycle.Initialisable;
 import org.mule.api.lifecycle.InitialisationException;
+import org.mule.api.lifecycle.Startable;
 import org.mule.api.processor.MessageProcessor;
+import org.mule.api.processor.MessageProcessorChain;
 import org.mule.config.i18n.CoreMessages;
 
 import java.util.concurrent.ConcurrentHashMap;
@@ -30,6 +33,9 @@ public class FlowRefFactoryBean
     implements FactoryBean<MessageProcessor>, ApplicationContextAware, MuleContextAware, Initialisable,
     Disposable
 {
+
+    private static final String NULL_FLOW_CONTRUCT_NAME = "null";
+    private static final String MULE_PREFIX = "_mule-";
     private String refName;
     private ApplicationContext applicationContext;
     private MuleContext muleContext;
@@ -81,8 +87,7 @@ public class FlowRefFactoryBean
         }
     }
 
-    protected MessageProcessor createDynamicReferenceMessageProcessor(String name)
-        throws InitialisationException
+    protected MessageProcessor createDynamicReferenceMessageProcessor(String name) throws MuleException
     {
         if (name == null)
         {
@@ -97,7 +102,7 @@ public class FlowRefFactoryBean
                 {
                     // Need to initialize because message processor won't be managed by parent
                     MessageProcessor dynamicMessageProcessor = getReferencedFlow(muleContext.getExpressionManager()
-                        .parse(refName, event));
+                        .parse(refName, event), event.getFlowConstruct());
                     return dynamicMessageProcessor.process(event);
                 }
             };
@@ -110,22 +115,47 @@ public class FlowRefFactoryBean
         return referenceCache.get(name);
     }
 
-    protected MessageProcessor getReferencedFlow(String name) throws InitialisationException
+    protected MessageProcessor getReferencedFlow(String name, FlowConstruct flowConstruct) throws MuleException
     {
         if (name == null)
         {
             throw new MuleRuntimeException(CoreMessages.objectIsNull(name));
         }
-        else if (!referenceCache.containsKey(name))
+        String categorizedName = getReferencedFlowCategorizedName(name, flowConstruct);
+        if (!referenceCache.containsKey(categorizedName))
         {
-            MessageProcessor processor = lookupReferencedFlowInApplicationContext(name);
-            if (processor instanceof Initialisable)
+            MessageProcessor referencedFlow = lookupReferencedFlowInApplicationContext(name);
+            if (referencedFlow instanceof Initialisable)
             {
-                ((Initialisable) processor).initialise();
+                if (referencedFlow instanceof FlowConstructAware)
+                {
+                    ((FlowConstructAware) referencedFlow).setFlowConstruct(flowConstruct);
+                }
+                if (referencedFlow instanceof MessageProcessorChain)
+                {
+                    for (MessageProcessor processor : ((MessageProcessorChain) referencedFlow).getMessageProcessors())
+                    {
+                        if (processor instanceof FlowConstructAware)
+                        {
+                            ((FlowConstructAware) processor).setFlowConstruct(flowConstruct);
+                        }
+                    }
+                }
+                ((Initialisable) referencedFlow).initialise();
             }
-            referenceCache.putIfAbsent(name, processor);
+            if (referencedFlow instanceof Startable)
+            {
+                ((Startable) referencedFlow).start();
+            }
+            referenceCache.putIfAbsent(categorizedName, referencedFlow);
         }
-        return referenceCache.get(name);
+        return referenceCache.get(categorizedName);
+    }
+
+    private String getReferencedFlowCategorizedName(String referencedFlowName, FlowConstruct flowConstruct)
+    {
+        String flowConstructName = flowConstruct != null ? flowConstruct.getName() : NULL_FLOW_CONTRUCT_NAME;
+        return MULE_PREFIX + flowConstructName + "-" + referencedFlowName;
     }
 
     protected MessageProcessor lookupReferencedFlowInApplicationContext(String name)
