@@ -1,15 +1,20 @@
 /*
- * $Id: DefaultLifecyclePhase.java 19191 2010-08-25 21:05:23Z tcarlson $
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.lifecycle.phases;
 
+import org.mule.api.MuleContext;
+import org.mule.api.context.MuleContextAware;
+import org.mule.api.lifecycle.LifecycleException;
+import org.mule.api.lifecycle.LifecyclePhase;
+import org.mule.api.lifecycle.LifecycleStateEnabled;
+import org.mule.config.ExceptionHelper;
+import org.mule.config.i18n.CoreMessages;
+import org.mule.lifecycle.LifecycleObject;
+
 import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -20,14 +25,6 @@ import java.util.Set;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-import org.mule.api.MuleContext;
-import org.mule.api.context.MuleContextAware;
-import org.mule.api.lifecycle.LifecycleException;
-import org.mule.api.lifecycle.LifecyclePhase;
-import org.mule.api.lifecycle.LifecycleStateEnabled;
-import org.mule.config.ExceptionHelper;
-import org.mule.config.i18n.CoreMessages;
-import org.mule.lifecycle.LifecycleObject;
 
 /**
  * Represents a configurable lifecycle phase. This is a default implementation of a
@@ -44,7 +41,7 @@ import org.mule.lifecycle.LifecycleObject;
  * {@link org.mule.api.lifecycle.LifecycleTransitionResult} and either 0 or 1
  * exceptions can be thrown which are either {@link InstantiationException} or
  * {@link org.mule.api.lifecycle.LifecycleException}.
- * 
+ *
  * @see org.mule.api.lifecycle.LifecyclePhase
  */
 public class DefaultLifecyclePhase implements LifecyclePhase, MuleContextAware
@@ -68,6 +65,7 @@ public class DefaultLifecyclePhase implements LifecyclePhase, MuleContextAware
         this.oppositeLifecyclePhase = oppositeLifecyclePhase;
     }
 
+    @Override
     public void setMuleContext(MuleContext context)
     {
         this.muleContext = context;
@@ -77,21 +75,23 @@ public class DefaultLifecyclePhase implements LifecyclePhase, MuleContextAware
      * Subclasses can override this method to order <code>objects</code> before the
      * lifecycle method is applied to them. This method does not apply any special
      * ordering to <code>objects</code>.
-     * 
+     *
      * @param objects
      * @param lo
      * @return List with ordered objects
      */
-    protected List sortLifecycleInstances(Collection objects, LifecycleObject lo)
+    protected List<?> sortLifecycleInstances(Collection<?> objects, LifecycleObject lo)
     {
-        return new ArrayList(objects);
+        return new ArrayList<Object>(objects);
     }
 
+    @Override
     public void addOrderedLifecycleObject(LifecycleObject lco)
     {
         orderedLifecycleObjects.add(lco);
     }
 
+    @Override
     public void removeOrderedLifecycleObject(LifecycleObject lco)
     {
         orderedLifecycleObjects.remove(lco);
@@ -117,51 +117,61 @@ public class DefaultLifecyclePhase implements LifecyclePhase, MuleContextAware
         return false;
     }
 
+    @Override
     public Set<LifecycleObject> getOrderedLifecycleObjects()
     {
         return orderedLifecycleObjects;
     }
 
+    @Override
     public void setOrderedLifecycleObjects(Set<LifecycleObject> orderedLifecycleObjects)
     {
         this.orderedLifecycleObjects = orderedLifecycleObjects;
     }
 
+    @Override
     public Class<?>[] getIgnoredObjectTypes()
     {
         return ignorredObjectTypes;
     }
 
+    @Override
     public void setIgnoredObjectTypes(Class<?>[] ignorredObjectTypes)
     {
         this.ignorredObjectTypes = ignorredObjectTypes;
     }
 
+    @Override
     public Class<?> getLifecycleClass()
     {
         return lifecycleClass;
     }
 
+    @Override
     public void setLifecycleClass(Class<?> lifecycleClass)
     {
         this.lifecycleClass = lifecycleClass;
     }
 
+    @Override
     public String getName()
     {
         return name;
     }
 
+    @Override
     public Set<String> getSupportedPhases()
     {
         return supportedPhases;
     }
 
+    @Override
     public void setSupportedPhases(Set<String> supportedPhases)
     {
         this.supportedPhases = supportedPhases;
     }
 
+    @Override
     public void registerSupportedPhase(String phase)
     {
         if (supportedPhases == null)
@@ -171,6 +181,7 @@ public class DefaultLifecyclePhase implements LifecyclePhase, MuleContextAware
         supportedPhases.add(phase);
     }
 
+    @Override
     public boolean isPhaseSupported(String phase)
     {
         if (getSupportedPhases() == null)
@@ -190,6 +201,7 @@ public class DefaultLifecyclePhase implements LifecyclePhase, MuleContextAware
         }
     }
 
+    @Override
     public void applyLifecycle(Object o) throws LifecycleException
     {
         if (o == null)
@@ -238,6 +250,7 @@ public class DefaultLifecyclePhase implements LifecyclePhase, MuleContextAware
         }
     }
 
+    @Override
     public String getOppositeLifecyclePhase()
     {
         return oppositeLifecyclePhase;
