@@ -1,15 +1,12 @@
 /*
- * $Id: MuleInvoker.java 24265 2012-04-16 06:54:00Z dfeist $
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.module.cxf;
 
+import org.mule.NonBlockingVoidMuleEvent;
 import org.mule.VoidMuleEvent;
 import org.mule.api.MuleEvent;
 import org.mule.api.MuleException;
@@ -18,6 +15,7 @@ import org.mule.api.config.MuleProperties;
 import org.mule.api.execution.ExecutionCallback;
 import org.mule.api.transport.PropertyScope;
 import org.mule.component.ComponentException;
+import org.mule.config.ExceptionHelper;
 import org.mule.execution.ErrorHandlingExecutionTemplate;
 import org.mule.transport.NullPayload;
 
@@ -25,6 +23,9 @@ import java.lang.reflect.Method;
 import java.lang.reflect.Proxy;
 import java.util.List;
 
+import javax.script.ScriptException;
+
+import org.apache.cxf.common.util.PropertyUtils;
 import org.apache.cxf.frontend.MethodDispatcher;
 import org.apache.cxf.helpers.CastUtils;
 import org.apache.cxf.interceptor.Fault;
@@ -55,69 +56,97 @@ public class MuleInvoker implements Invoker
         // this is the original request. Keep it to copy all the message properties from it
         final MuleEvent event = (MuleEvent) exchange.get(CxfConstants.MULE_EVENT);
         MuleEvent responseEvent = null;
-        try
+
+        if (PropertyUtils.isTrue(exchange.remove(CxfConstants.NON_BLOCKING_RESPONSE)))
         {
-            MuleMessage reqMsg = event.getMessage();
-            reqMsg.setPayload(extractPayload(exchange.getInMessage()));
-            
-            BindingOperationInfo bop = exchange.get(BindingOperationInfo.class);
-            Service svc = exchange.get(Service.class);
-            if (!cxfMmessageProcessor.isProxy())
+            responseEvent = event;
+        }
+        else
+        {
+            try
             {
-                MethodDispatcher md = (MethodDispatcher) svc.get(MethodDispatcher.class.getName());
-                Method m = md.getMethod(bop);
-                if (targetClass != null)
+                MuleMessage reqMsg = event.getMessage();
+                reqMsg.setPayload(extractPayload(exchange.getInMessage()));
+
+                BindingOperationInfo bop = exchange.get(BindingOperationInfo.class);
+                Service svc = exchange.get(Service.class);
+                if (!cxfMmessageProcessor.isProxy())
                 {
-                    m = matchMethod(m, targetClass);
+                    MethodDispatcher md = (MethodDispatcher) svc.get(MethodDispatcher.class.getName());
+                    Method m = md.getMethod(bop);
+                    if (targetClass != null)
+                    {
+                        m = matchMethod(m, targetClass);
+                    }
+
+                    reqMsg.setProperty(MuleProperties.MULE_METHOD_PROPERTY, m, PropertyScope.INVOCATION);
                 }
-            
-                reqMsg.setProperty(MuleProperties.MULE_METHOD_PROPERTY, m, PropertyScope.INVOCATION);
-            }
 
-            if (bop != null)
-            {
-                reqMsg.setProperty(CxfConstants.INBOUND_OPERATION, bop.getOperationInfo().getName(), PropertyScope.INVOCATION);
-                reqMsg.setProperty(CxfConstants.INBOUND_SERVICE, svc.getName(), PropertyScope.INVOCATION);
-            }
-
-            ErrorHandlingExecutionTemplate errorHandlingExecutionTemplate = ErrorHandlingExecutionTemplate.createErrorHandlingExecutionTemplate(event.getMuleContext(), event.getFlowConstruct().getExceptionListener());
-            responseEvent = errorHandlingExecutionTemplate.execute(new ExecutionCallback<MuleEvent>()
-            {
-                @Override
-                public MuleEvent process() throws Exception
+                if (bop != null)
                 {
-                    return cxfMmessageProcessor.processNext(event);
+                    reqMsg.setProperty(CxfConstants.INBOUND_OPERATION, bop.getOperationInfo().getName(), PropertyScope.INVOCATION);
+                    reqMsg.setProperty(CxfConstants.INBOUND_SERVICE, svc.getName(), PropertyScope.INVOCATION);
                 }
-            });
-        }
-        catch (MuleException e)
-        {
-            exchange.put(CxfConstants.MULE_EVENT, event);
 
-            Throwable cause = e;
-            if (e instanceof ComponentException) {
-                cause = e.getCause();
+                ErrorHandlingExecutionTemplate errorHandlingExecutionTemplate = ErrorHandlingExecutionTemplate.createErrorHandlingExecutionTemplate(event.getMuleContext(), event.getFlowConstruct().getExceptionListener());
+                responseEvent = errorHandlingExecutionTemplate.execute(new ExecutionCallback<MuleEvent>()
+                {
+                    @Override
+                    public MuleEvent process() throws Exception
+                    {
+                        return cxfMmessageProcessor.processNext(event);
+                    }
+                });
             }
-            throw new Fault(cause);
-        }
-        catch (Exception e)
-        {
-            exchange.put(CxfConstants.MULE_EVENT, event);
-            throw new Fault(e);
-        }
+            catch (MuleException e)
+            {
+                exchange.put(CxfConstants.MULE_EVENT, event);
 
+                Throwable cause = e;
+
+                // See MULE-6329
+                if(Boolean.valueOf((String) event.getMessage().getInvocationProperty(CxfConstants.UNWRAP_MULE_EXCEPTIONS)))
+                {
+                    cause = ExceptionHelper.getNonMuleException(e);
+                    // Exceptions thrown from a ScriptComponent or a ScriptTransformer are going to be wrapped on a
+                    // ScriptException
+                    if(cause instanceof ScriptException && cause.getCause() != null)
+                    {
+                        cause = cause.getCause();
+                    }
+                }
+                else if(e instanceof ComponentException) {
+                    cause = e.getCause();
+                }
+
+                throw new Fault(cause);
+            }
+            catch (Exception e)
+            {
+                exchange.put(CxfConstants.MULE_EVENT, event);
+                throw new Fault(e);
+            }
+
+            if (!event.getExchangePattern().hasResponse())
+            {
+                // weird response from AbstractInterceptingMessageProcessor
+                responseEvent = null;
+            }
+
+            if (responseEvent instanceof NonBlockingVoidMuleEvent)
+            {
+                exchange.put(Message.SUSPENDED_INVOCATION, true);
+                exchange.put(CxfConstants.MULE_EVENT, responseEvent);
+                return null;
+            }
 
-        if (!event.getExchangePattern().hasResponse())
-        {
-            // weird response from AbstractInterceptingMessageProcessor
-            responseEvent = null;
         }
 
         if (responseEvent != null && !VoidMuleEvent.getInstance().equals(responseEvent))
         {
             exchange.put(CxfConstants.MULE_EVENT, responseEvent);
             MuleMessage resMessage = responseEvent.getMessage();
-            
+
             if (resMessage.getExceptionPayload() != null)
             {
                 Throwable cause = resMessage.getExceptionPayload().getException();
@@ -159,7 +188,7 @@ public class MuleInvoker implements Invoker
             return null;
         }
     }
-    
+
     protected Object extractPayload(Message cxfMessage)
     {   
         List<Object> list = CastUtils.cast(cxfMessage.getContent(List.class));
