@@ -7,11 +7,12 @@
 package org.mule.module.http.api.listener;
 
 import org.mule.api.MuleException;
+import org.mule.api.config.Config;
 import org.mule.api.lifecycle.Startable;
 import org.mule.api.lifecycle.Stoppable;
 import org.mule.transport.ssl.api.TlsContextFactory;
 
-public interface HttpListenerConfig extends Startable, Stoppable
+public interface HttpListenerConfig extends Config, Startable, Stoppable
 {
 
     /**
