@@ -6,6 +6,7 @@
  */
 package org.mule.module.launcher.application;
 
+import org.mule.module.launcher.GoodCitizenClassLoader;
 import org.mule.module.launcher.MuleApplicationClassLoader;
 import org.mule.module.launcher.nativelib.NativeLibraryFinderFactory;
 import org.mule.module.launcher.artifact.ArtifactClassLoader;
@@ -14,6 +15,7 @@ import org.mule.module.launcher.domain.DomainClassLoaderRepository;
 import org.mule.module.launcher.plugin.MulePluginsClassLoader;
 import org.mule.module.launcher.plugin.PluginDescriptor;
 
+import java.net.URL;
 import java.util.Set;
 
 /**
@@ -48,6 +50,13 @@ public class MuleApplicationClassLoaderFactory implements ApplicationClassLoader
         final Set<PluginDescriptor> plugins = descriptor.getPlugins();
         if (!plugins.isEmpty())
         {
+            // Re-assigns parent if there are shared plugin libraries
+            URL[] pluginLibs = descriptor.getSharedPluginLibs();
+            if (pluginLibs != null && pluginLibs.length != 0)
+            {
+                parent = new GoodCitizenClassLoader(pluginLibs, parent);
+            }
+
             // re-assign parent ref if any plugins deployed, will be used by the MuleAppCL
             parent = new MulePluginsClassLoader(parent, plugins);
         }
