@@ -1,8 +1,5 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
@@ -27,27 +24,32 @@ import org.mule.api.transport.Connector;
 import org.mule.config.i18n.MessageFactory;
 import org.mule.lifecycle.phases.NotInLifecyclePhase;
 import org.mule.util.CollectionUtils;
+import org.mule.util.ExceptionUtils;
 import org.mule.util.StringUtils;
 
 import java.util.Collection;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
+import java.util.TreeSet;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReadWriteLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import org.apache.commons.collections.Predicate;
 import org.apache.commons.collections.functors.InstanceofPredicate;
-import org.apache.commons.logging.Log;
+import org.slf4j.Logger;
 
 /**
  * Use the registryLock when reading/writing/iterating over the contents of the registry hashmap.
+ * @deprecated as of 3.7.0. Use {@link SimpleRegistry instead}.
  */
-//@ThreadSafe
+@Deprecated
 public class TransientRegistry extends AbstractRegistry
 {
+
     public static final String REGISTRY_ID = "org.mule.Registry.Transient";
 
     private final RegistryMap registryMap = new RegistryMap(logger);
@@ -65,7 +67,7 @@ public class TransientRegistry extends AbstractRegistry
 
     private void putDefaultEntriesIntoRegistry()
     {
-        Map<String, Object> processors = new HashMap<String, Object>();
+        Map<String, Object> processors = new HashMap<>();
         processors.put("_muleContextProcessor", new MuleContextProcessor(muleContext));
         //processors("_muleNotificationProcessor", new NotificationListenersProcessor(muleContext));
         processors.put("_muleExpressionEvaluatorProcessor", new ExpressionEvaluatorProcessor(muleContext));
@@ -90,17 +92,40 @@ public class TransientRegistry extends AbstractRegistry
     @Override
     protected void doDispose()
     {
+        disposeLostObjects();
         registryMap.clear();
     }
 
+    private void disposeLostObjects()
+    {
+        for (Object obj : registryMap.getLostObjects())
+        {
+            if (obj != null && obj instanceof Disposable)
+            {
+                try
+                {
+                    ((Disposable) obj).dispose();
+                }
+                catch (Exception e)
+                {
+                    logger.warn("Can not dispose object. " + ExceptionUtils.getMessage(e));
+                    if (logger.isDebugEnabled())
+                    {
+                        logger.debug("Can not dispose object. " + ExceptionUtils.getFullStackTrace(e));
+                    }
+                }
+            }
+        }
+    }
+
     protected Map<String, Object> applyProcessors(Map<String, Object> objects)
     {
-        if (objects == null)
+        if (objects == null || !isInitialised())
         {
             return null;
         }
 
-        Map<String, Object> results = new HashMap<String, Object>();
+        Map<String, Object> results = new HashMap<>();
         for (Map.Entry<String, Object> entry : objects.entrySet())
         {
             // We do this inside the loop in case the map contains ObjectProcessors
@@ -158,7 +183,13 @@ public class TransientRegistry extends AbstractRegistry
 
     public <T> T lookupObject(String key)
     {
-        return registryMap.<T>get(key);
+        return doGet(key);
+    }
+
+    @Override
+    public <T> T lookupObject(Class<T> type) throws RegistrationException
+    {
+        return super.lookupObject(type);
     }
 
     @SuppressWarnings("unchecked")
@@ -167,6 +198,13 @@ public class TransientRegistry extends AbstractRegistry
         return (Collection<T>) registryMap.select(new InstanceofPredicate(returntype));
     }
 
+    @Override
+    public <T> Collection<T> lookupLocalObjects(Class<T> type)
+    {
+        //just delegate to lookupObjects since there's no parent ever
+        return lookupObjects(type);
+    }
+
     /**
      * Will fire any lifecycle methods according to the current lifecycle without actually
      * registering the object in the registry.  This is useful for prototype objects that are created per request and would
@@ -188,11 +226,16 @@ public class TransientRegistry extends AbstractRegistry
         return object;
     }
 
-    Object applyProcessors(Object object, Object metadata)
+    protected Object applyProcessors(Object object, Object metadata)
     {
+        if (!isInitialised())
+        {
+            return object;
+        }
+
         Object theObject = object;
 
-        if(!hasFlag(metadata, MuleRegistry.INJECT_PROCESSORS_BYPASS_FLAG))
+        if (!hasFlag(metadata, MuleRegistry.INJECT_PROCESSORS_BYPASS_FLAG))
         {
             //Process injectors first
             Collection<InjectProcessor> injectProcessors = lookupObjects(InjectProcessor.class);
@@ -202,14 +245,14 @@ public class TransientRegistry extends AbstractRegistry
             }
         }
 
-        if(!hasFlag(metadata, MuleRegistry.PRE_INIT_PROCESSORS_BYPASS_FLAG))
+        if (!hasFlag(metadata, MuleRegistry.PRE_INIT_PROCESSORS_BYPASS_FLAG))
         {
             //Then any other processors
             Collection<PreInitProcessor> processors = lookupObjects(PreInitProcessor.class);
             for (PreInitProcessor processor : processors)
             {
                 theObject = processor.process(theObject);
-                if(theObject==null)
+                if (theObject == null)
                 {
                     return null;
                 }
@@ -252,13 +295,23 @@ public class TransientRegistry extends AbstractRegistry
             return;
         }
 
-        registryMap.putAndLogWarningIfDuplicate(key, object);
+        doRegisterObject(key, object, metadata);
+    }
+
+    protected <T> T doGet(String key)
+    {
+        return registryMap.get(key);
+    }
+
+    protected void doRegisterObject(String key, Object object, Object metadata) throws RegistrationException
+    {
+        doPut(key, object);
 
         try
         {
             if (!hasFlag(metadata, MuleRegistry.LIFECYCLE_BYPASS_FLAG))
             {
-                if(logger.isDebugEnabled())
+                if (logger.isDebugEnabled())
                 {
                     logger.debug("applying lifecycle to object: " + object);
                 }
@@ -271,9 +324,14 @@ public class TransientRegistry extends AbstractRegistry
         }
     }
 
+    protected void doPut(String key, Object object)
+    {
+        registryMap.putAndLogWarningIfDuplicate(key, object);
+    }
+
     protected void checkDisposed() throws RegistrationException
     {
-        if(getLifecycleManager().isPhaseComplete(Disposable.PHASE_NAME))
+        if (getLifecycleManager().isPhaseComplete(Disposable.PHASE_NAME))
         {
             throw new RegistrationException(MessageFactory.createStaticMessage("Cannot register objects on the registry as the context is disposed"));
         }
@@ -284,36 +342,10 @@ public class TransientRegistry extends AbstractRegistry
         return !(metaData == null || !(metaData instanceof Integer)) && ((Integer) metaData & flag) != 0;
     }
 
-    /**
-     * Will remove an object by name from the registry. By default the registry will apply all remaining lifecycle phases
-     * to the object when it is removed.
-     *
-     * @param key the name or key of the object to remove from the registry
-     * @param metadata Meta data flags supported are {@link org.mule.api.registry.MuleRegistry#LIFECYCLE_BYPASS_FLAG}
-     * @throws RegistrationException if there is a problem unregistering the object. Typically this will be because
-     * the object's lifecycle threw an exception
-     */
-    public void unregisterObject(String key, Object metadata) throws RegistrationException
+    @Override
+    protected Object doUnregisterObject(String key) throws RegistrationException
     {
-        Object obj = registryMap.remove(key);
-
-        try
-        {
-            if (!hasFlag(metadata, MuleRegistry.LIFECYCLE_BYPASS_FLAG))
-            {
-                getLifecycleManager().applyPhase(obj, lifecycleManager.getCurrentPhase(), Disposable.PHASE_NAME);
-            }
-        }
-        catch (MuleException e)
-        {
-            throw new RegistrationException(e);
-        }
-
-    }
-
-    public void unregisterObject(String key) throws RegistrationException
-    {
-        unregisterObject(key, Object.class);
+        return registryMap.remove(key);
     }
 
     // /////////////////////////////////////////////////////////////////////////
@@ -335,14 +367,28 @@ public class TransientRegistry extends AbstractRegistry
      * transient registry and also shields client code from having to deal with locking the
      * {@link ReadWriteLock} for the exposed Map operations.
      */
-    private static class RegistryMap
+    protected static class RegistryMap
     {
+
         private final Map<String, Object> registry = new HashMap<String, Object>();
         private final ReadWriteLock registryLock = new ReentrantReadWriteLock();
+        private final Set<Object> lostObjects = new TreeSet<Object>(new Comparator<Object>()
+        {
+            @Override
+            public int compare(Object o1, Object o2)
+            {
+                return o1 == o2 ? 0 : nvl(o1) - nvl(o2);
+            }
 
-        private Log logger;
+            private int nvl(Object o)
+            {
+                return o != null ? o.hashCode() : 0;
+            }
+        });
 
-        public RegistryMap(Log log)
+        private Logger logger;
+
+        public RegistryMap(Logger log)
         {
             super();
             logger = log;
@@ -369,6 +415,7 @@ public class TransientRegistry extends AbstractRegistry
             {
                 writeLock.lock();
                 registry.clear();
+                lostObjects.clear();
             }
             finally
             {
@@ -383,14 +430,15 @@ public class TransientRegistry extends AbstractRegistry
             {
                 writeLock.lock();
 
-                if (registry.containsKey(key))
+                final Object previousObject = registry.put(key, object);
+                if (previousObject != null && previousObject != object)
                 {
+                    lostObjects.add(previousObject);
                     // registry.put(key, value) would overwrite a previous entity with the same name.  Is this really what we want?
                     // Not sure whether to throw an exception or log a warning here.
                     //throw new RegistrationException("TransientRegistry already contains an object named '" + key + "'.  The previous object would be overwritten.");
                     logger.warn("TransientRegistry already contains an object named '" + key + "'.  The previous object will be overwritten.");
                 }
-                registry.put(key, object);
             }
             finally
             {
@@ -446,6 +494,11 @@ public class TransientRegistry extends AbstractRegistry
             return registry.entrySet();
         }
 
+        public Set<Object> getLostObjects()
+        {
+            return lostObjects;
+        }
+
         public void lockForReading()
         {
             registryLock.readLock().lock();
