@@ -1,13 +1,9 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.transformer;
 
 import org.mule.api.DefaultMuleException;
@@ -24,22 +20,22 @@ import org.mule.transport.NullPayload;
 import org.mule.transport.service.TransportFactoryException;
 import org.mule.transport.service.TransportServiceDescriptor;
 import org.mule.util.ClassUtils;
+import org.mule.util.StringUtils;
 
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.StringTokenizer;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 public class TransformerUtils
 {
 
+    private static Logger LOGGER = LoggerFactory.getLogger(AbstractTransformer.class);
     public static final String COMMA = ",";
 
-    private static Log logger = LogFactory.getLog(AbstractTransformer.class);
-
     public static void initialiseAllTransformers(List<Transformer> transformers) throws InitialisationException
     {
         if (transformers != null)
@@ -54,7 +50,7 @@ public class TransformerUtils
 
     public static String toString(List<Transformer> transformers)
     {
-        StringBuffer buffer = new StringBuffer();
+        StringBuilder buffer = new StringBuilder();
         Iterator<Transformer> transformer = transformers.iterator();
         while (transformer.hasNext())
         {
@@ -100,7 +96,7 @@ public class TransformerUtils
         }
         catch (MuleException e)
         {
-            logger.debug(e.getMessage(), e);
+            LOGGER.debug(e.getMessage(), e);
             return null;
         }
     }
@@ -196,9 +192,75 @@ public class TransformerUtils
             }
         }
 
-        if (logger.isDebugEnabled())
+        if (LOGGER.isDebugEnabled())
         {
-            logger.debug("The transformed value is of expected type. Type is: " + ClassUtils.getSimpleName(value.getClass()));
+            LOGGER.debug("The transformed value is of expected type. Type is: " + ClassUtils.getSimpleName(value.getClass()));
         }
     }
+
+    public static <T> Object transformToAny(T input, MuleContext muleContext, DataType<?>... supportedTypes)
+    {
+        final DataType sourceType = DataTypeFactory.create(input.getClass());
+        Object transformedData = null;
+
+        for (DataType<?> supportedType : supportedTypes)
+        {
+            transformedData = attemptTransformation(sourceType, input, supportedType, muleContext);
+            if (transformedData != null)
+            {
+                break;
+            }
+        }
+
+        return transformedData;
+    }
+
+    private static <S, R> R attemptTransformation(DataType<S> sourceDataType, S source, DataType<R> resultDataType, MuleContext muleContext)
+    {
+        Transformer transformer;
+        try
+        {
+            transformer = muleContext.getRegistry().lookupTransformer(sourceDataType, resultDataType);
+        }
+        catch (TransformerException e)
+        {
+            LOGGER.debug("Could not find a transformer from type {} to {}", sourceDataType.getType().getName(), resultDataType.getType().getName());
+            return null;
+        }
+
+        LOGGER.debug("Located transformer {} from type {} to type {}. Attempting transformation...", transformer.getName(), sourceDataType.getType().getName(), resultDataType.getType().getName());
+
+        try
+        {
+            return (R) transformer.transform(source);
+        }
+        catch (TransformerException e)
+        {
+            if (LOGGER.isDebugEnabled())
+            {
+                LOGGER.debug(
+                        String.format("Transformer %s threw exception while trying to transform an object of type %s into a %s",
+                                      transformer.getName(), sourceDataType.getType().getName(), resultDataType.getType().getName())
+                        , e);
+            }
+
+            return null;
+        }
+    }
+
+    public static String generateTransformerName(Class<? extends Transformer> transformerClass, DataType returnType)
+    {
+        String transformerName = ClassUtils.getSimpleName(transformerClass);
+        int i = transformerName.indexOf("To");
+        if (i > 0 && returnType != null)
+        {
+            String target = ClassUtils.getSimpleName(returnType.getType());
+            if (target.equals("byte[]"))
+            {
+                target = "byteArray";
+            }
+            transformerName = transformerName.substring(0, i + 2) + StringUtils.capitalize(target);
+        }
+        return transformerName;
+    }
 }
