@@ -1,20 +1,15 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule.transformer.simple;
 
 import org.mule.api.transformer.DiscoverableTransformer;
 import org.mule.api.transformer.TransformerException;
 import org.mule.transformer.AbstractTransformer;
 import org.mule.transformer.types.DataTypeFactory;
-import org.mule.util.SerializationUtils;
 
 import java.io.Serializable;
 
@@ -64,7 +59,7 @@ public class SerializableToByteArray extends AbstractTransformer implements Disc
 
         try
         {
-            return SerializationUtils.serialize((Serializable) src);
+            return muleContext.getObjectSerializer().serialize(src);
         }
         catch (Exception e)
         {
