@@ -283,20 +283,21 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
             logger.info("Lock obtained on file: " + file.getAbsolutePath());
         }
 
+        // The file may get moved/renamed here so store the original file info.
+        final String originalSourceFilePath = file.getAbsolutePath();
+        final String originalSourceFileName = file.getName();
+        final String originalSourceDirectory = file.getParent();
+
         // This isn't nice but is needed as MuleMessage is required to resolve
         // destination file name
         DefaultMuleMessage fileParserMessasge = new DefaultMuleMessage(null, getEndpoint().getMuleContext());
-        fileParserMessasge.setInboundProperty(FileConnector.PROPERTY_ORIGINAL_FILENAME, file.getName());
+        fileParserMessasge.setInboundProperty(FileConnector.PROPERTY_ORIGINAL_FILENAME, originalSourceFileName);
+        fileParserMessasge.setInboundProperty(FileConnector.PROPERTY_ORIGINAL_DIRECTORY, originalSourceDirectory);
 
-        // The file may get moved/renamed here so store the original file info.
-        final String originalSourceFile = file.getAbsolutePath();
-        final String originalSourceFileName = file.getName();
         final File sourceFile;
         if (workDir != null)
         {
-            String workFileName = file.getName();
-
-            workFileName = fileConnector.getFilenameParser().getFilename(fileParserMessasge, workFileNamePattern);
+            String workFileName = fileConnector.getFilenameParser().getFilename(fileParserMessasge, workFileNamePattern);
             // don't use new File() directly, see MULE-1112
             File workFile = FileUtils.newFile(workDir, workFileName);
 
@@ -316,8 +317,7 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
             String destinationFileName = originalSourceFileName;
             if (moveToPattern != null)
             {
-                destinationFileName = fileConnector.getFilenameParser().getFilename(fileParserMessasge,
-                                                                                    moveToPattern);
+                destinationFileName = fileConnector.getFilenameParser().getFilename(fileParserMessasge, moveToPattern);
             }
             // don't use new File() directly, see MULE-1112
             destinationFile = FileUtils.newFile(moveDir, destinationFileName);
@@ -357,7 +357,12 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
             message.setProperty(FileConnector.PROPERTY_SOURCE_FILENAME, file.getName(), PropertyScope.INBOUND);
         }
 
+        message.setProperty(FileConnector.PROPERTY_ORIGINAL_DIRECTORY, originalSourceDirectory, PropertyScope.INBOUND);
+        message.setProperty(FileConnector.PROPERTY_ORIGINAL_FILENAME, originalSourceFileName, PropertyScope.INBOUND);
+
+        message.setInvocationProperty(FileConnector.PROPERTY_ORIGINAL_DIRECTORY, originalSourceDirectory);
         message.setInvocationProperty(FileConnector.PROPERTY_ORIGINAL_FILENAME, originalSourceFileName);
+
         if (forceSync)
         {
             message.setProperty(MuleProperties.MULE_FORCE_SYNC_PROPERTY, Boolean.TRUE, PropertyScope.INBOUND);
@@ -374,7 +379,7 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
         }
         else
         {
-            processWithoutStreaming(originalSourceFile, originalSourceFileName, sourceFile, destinationFile, executionTemplate, finalMessage);
+            processWithoutStreaming(originalSourceFilePath, originalSourceFileName, originalSourceDirectory, sourceFile, destinationFile, executionTemplate, finalMessage);
         }
     }
 
@@ -421,7 +426,7 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
         }
     }
 
-    private void processWithoutStreaming(String originalSourceFile, final String originalSourceFileName, final File sourceFile,final File destinationFile, ExecutionTemplate<MuleEvent> executionTemplate, final MuleMessage finalMessage) throws DefaultMuleException
+    private void processWithoutStreaming(String originalSourceFile, final String originalSourceFileName, final String originalSourceDirectory, final File sourceFile,final File destinationFile, ExecutionTemplate<MuleEvent> executionTemplate, final MuleMessage finalMessage) throws DefaultMuleException
     {
         try
         {
@@ -430,7 +435,7 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
                 @Override
                 public MuleEvent process() throws Exception
                 {
-                    moveAndDelete(sourceFile, destinationFile, originalSourceFileName, finalMessage);
+                    moveAndDelete(sourceFile, destinationFile, originalSourceFileName, originalSourceDirectory, finalMessage);
                     return null;
                 }
             });
@@ -543,7 +548,7 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
     }
 
     private void moveAndDelete(final File sourceFile, File destinationFile,
-                               String sourceFileOriginalName, MuleMessage message) throws MuleException
+                               String originalSourceFileName, String originalSourceDirectory, MuleMessage message) throws MuleException
     {
         // If we are moving the file to a read directory, move it there now and
         // hand over a reference to the
@@ -565,7 +570,8 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
             // create new Message for destinationFile
             message = createMuleMessage(destinationFile, endpoint.getEncoding());
             message.setProperty(FileConnector.PROPERTY_FILENAME, destinationFile.getName(), PropertyScope.INBOUND);
-            message.setProperty(FileConnector.PROPERTY_ORIGINAL_FILENAME, sourceFileOriginalName, PropertyScope.INBOUND);
+            message.setProperty(FileConnector.PROPERTY_ORIGINAL_FILENAME, originalSourceFileName, PropertyScope.INBOUND);
+            message.setProperty(FileConnector.PROPERTY_ORIGINAL_DIRECTORY, originalSourceDirectory, PropertyScope.INBOUND);
         }
 
         // finally deliver the file message
@@ -682,15 +688,7 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
 
     protected void basicListFiles(File currentDirectory, List<File> discoveredFiles)
     {
-        File[] files;
-        if (fileFilter != null)
-        {
-            files = currentDirectory.listFiles(fileFilter);
-        }
-        else
-        {
-            files = currentDirectory.listFiles(filenameFilter);
-        }
+        File[] files = currentDirectory.listFiles();
 
         // the listFiles calls above may actually return null (check the JDK code).
         if (files == null)
@@ -700,15 +698,29 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
 
         for (File file : files)
         {
-            if (!file.isDirectory())
-            {
-                discoveredFiles.add(file);
-            }
-            else
+            if (file.isDirectory())
             {
                 if (fileConnector.isRecursive())
                 {
-                    this.basicListFiles(file, discoveredFiles);
+                    basicListFiles(file, discoveredFiles);
+                }
+            }
+            else
+            {
+                boolean addFile = true;
+
+                if (fileFilter != null)
+                {
+                    addFile = fileFilter.accept(file);
+                }
+                else if (filenameFilter != null)
+                {
+                    addFile = filenameFilter.accept(currentDirectory, file.getName());
+                }
+
+                if (addFile)
+                {
+                    discoveredFiles.add(file);
                 }
             }
         }
@@ -744,7 +756,7 @@ public class FileMessageReceiver extends AbstractPollingMessageReceiver
                 reverse = Boolean.valueOf((String) reverseProperty);
             }
 
-            Class<?> clazz = Class.forName(comparatorClassName.toString());
+            Class<?> clazz = endpoint.getMuleContext().getExecutionClassLoader().loadClass(comparatorClassName.toString());
             Comparator<?> comparator = (Comparator<?>)clazz.newInstance();
             return reverse ? new ReverseComparator(comparator) : comparator;
         }
